#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup


setup(name='fastuaparser',
      version='0.1.3',
      description='A super-fast user agent string parser',
      author='Almar Klein',
      author_email='almar.klein@gmail.com',
      url='https://github.com/almarklein/fastuaparser',
      py_modules=['fastuaparser'],
     )
